import "dart:async";
import 'dart:typed_data';
import 'package:dvote/util/content-uri.dart';
import 'package:convert/convert.dart';
import 'package:dvote/util/parsers.dart';

import '../blockchain/index.dart';
import '../models/dart/entity.pb.dart';
import './file.dart';

Future<Entity> fetchEntity(String entityId, String entityResolverAddress,
    String dvoteGatewayUri, String web3GatewayUri,
    {String gatewayPublicKey,
    String networkId,
    List<String> entryPoints}) async {
  // Fetch the Content URI from the blockchain
  final hexEntityId = hex.decode(entityId.substring(2));
  final List<dynamic> result = await callEntityMethod(
      web3GatewayUri,
      entityResolverAddress,
      "text",
      [Uint8List.fromList(hexEntityId), "vnd.vocdoni.meta"]);

  if (result == null || result.length == 0 || result.first == null)
    throw "The metadata of the entity can't be found";
  else if (!(result[0] is String))
    throw "The response from the blockchain is invalid";

  // Fetch the JSON from the network
  final contentUri = ContentURI(result[0]);
  final meta = await fetchFileString(contentUri, dvoteGatewayUri,
      gatewayPublicKey: gatewayPublicKey);

  try {
    final entity = parseEntity(meta);
    return entity;
  } catch (err) {
    throw "The JSON metadata is not valid";
  }
}
