import 'package:dvote/util/parsers.dart';
import 'package:flutter/services.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:dvote/dvote.dart';
import 'package:dvote/api/entity.dart';
import 'package:dvote/api/file.dart';
import 'package:dvote/util/content-uri.dart';

void main() {
  hdWallet();
  vocGateway();
  web3Gateway();
  dataModels();
}

void hdWallet() {
  const MethodChannel channel = MethodChannel('dvote');

  setUp(() {
    channel.setMockMethodCallHandler((MethodCall methodCall) async {
      return '0xbf0177fac1aad0cba87e5c0f54742fae36d69fff';
    });
  });

  tearDown(() {
    channel.setMockMethodCallHandler(null);
  });

  test('mnemonicToAddress', () async {
    expect(
        await mnemonicToAddress(
            'master lift organ scale accuse under head grace deny panic apart check',
            hdPath: "m/44'/60'/0'/0/0"),
        '0xbf0177fac1aad0cba87e5c0f54742fae36d69fff');
  });
}

void vocGateway() {
  final origin = "ipfs://Qmaisz6NMhDB51cCvNWa1GMS7LU1pAxdF4Ld6Ft9kZEP2a";
  final gwUri = "ws://gateway:2082/dvote";
  final publicKey =
      "02325f284f50fa52d53579c7873a480b351cc20f7780fa556929f5017283ad2449";

  test('fetchFile', () async {
    final res = await fetchFileString(ContentURI(origin), gwUri,
        gatewayPublicKey: publicKey);
    expect(res, "Hello from IPFS Gateway Checker\n");
  });
}

void web3Gateway() {
  test('web3Gateway', () async {
    final result = await fetchEntity(
      "0x180dd5765d9f7ecef810b565a2e5bd14a3ccd536c442b3de74867df552855e85",
      "0x0dCA233CE5152d58c74E74693A3C496D01542244",
      "ws://gateway:2082/dvote",
      "http://gateway:2086/web3",
      networkId: "1234",
      entryPoints: [],
    );

    print(result.writeToJson());
  });
}

void dataModels() {
  test("Entity model", () async {
    final meta =
        """{"version":"1.0","entityId":"0x409568155b5da230b6d95ca9460713b3c4d12c59851ed84aaa7ba52844c0eb5f","languages":["default"],"name":{"default":"My official entity","fr":"Mon organisation officielle"},"description":{"default":"The description of my entity goes here","fr":"La description officielle de mon organisation est ici"},"contracts":{"resolverAddress":"0x21f7DcCd9D1ce4C3685A5c50096265A8db4103b4","votingAddress":"0x1234567890123456789012345678901234567890","networkId":"goerli"},"votingProcesses":{"active":[],"ended":[]},"newsFeed":{"default":"https://hipsterpixel.co/feed.json","fr":"https://feed2json.org/convert?url=http://www.intertwingly.net/blog/index.atom"},"media":{"avatar":"https://hipsterpixel.co/assets/favicons/apple-touch-icon.png","header":"https://images.unsplash.com/photo-1557518016-299b3b3c2e7f?ixlib=rb-1.2.1&ixid=eyJhcHBfaWQiOjEyMDd9&auto=format&fit=crop&w=500&q=80"},"actions":[{"type":"browser","name":{"default":"Sign up to The Entity","fr":"S'inscrire à l'organisation"},"url":"https://cloudflare-ipfs.com/ipfs/QmZ56Z2kpG5QjJcWfhxFD4ac3DhfX21hrQ2gCTrWxzTAse","visible":"always"}],"gatewayBootNodes":[{"fetchUri":"https://bootnode:port/gateways.json","heartbeatMessagingUri":"pss://publicKey@0x0"}],"gatewayUpdate":{"timeout":60000,"topic":"vocdoni-gateway-update","difficulty":1000},"relays":[{"publicKey":"04875204f7b0bd9dfcf9af89b4fa0c44016f4b0372646c2142f55136973bd0d660565755341e4c583d49f9d607d08d40e3558de7e6f14f34aa83b7436ac70dc958","messagingUri":"<messaging-uri>"}],"bootEntities":[],"fallbackBootNodeEntities":[],"trustedEntities":[],"censusServiceManagedEntities":[]}""";
    Entity entity = parseEntity(meta);
    expect(entity.name["default"], "My official entity",
        reason: "The name should equal My official entity");
  });
  test("Feed model", () async {
    final meta = '''{
  "version": "https://jsonfeed.org/version/1",
  "title": "My Entity Feed",
  "home_page_url": "https://hipsterpixel.co/",
  "description": "I am the description of the entity",
  "feed_url": "https://hipsterpixel.co/feed.json",
  "icon": "https://hipsterpixel.co/assets/favicons/apple-touch-icon.png",
  "favicon": "https://hipsterpixel.co/assets/favicons/favicon.ico",
  "expired": false,
  "items": [
    {
      "id": "900e5aa6896c53a40745acac8ca00c3c0ae4f7c3",
      "title": "China's latest weapon in the trade war: Karaoke",
      "summary": "A Chinese propaganda song about the ongoing Sino-US trade war is getting a lot of interest - and raising a few eyebrows - on Chinese social media.",
      "content_text": "Many cameras nowadays come with a nice screen, often high resolution with a high brightness, but not always swivelling and always too small to fully rely on. I cannot tell how many times a bad picture on the small camera display was actually pretty decent once I opened it on the computer. And when you’re doing video, you really need to get that focus right, at all times. This is hard, but I have a solution that will help you out tremendously!",
      "content_html": "<h1>I'm an H1</h1> <h2>I'm an H2</h2> <h3>I'm an H3</h3> <img src=\\"https://i.udemycdn.com/course/750x422/59535_1f48_6.jpg\\" alt=\\"Girl in a jacket\\">",
      "url": "https://hipsterpixel.co/2019/05/10/smallhd-5-5-focus-oled-monitor-review/",
      "image": "https://ichef.bbci.co.uk/news/768/cpsprodpb/E24F/production/_107053975_tradewar.png",
      "tags": [
        "smallhd",
        "oled",
        "monitor",
        "camera",
        "review",
        "test",
        "videography",
        "photography",
        "filmmaking",
        "sonycamera",
        "workflow",
        "gearhead"
      ],
      "date_published": "2019-05-10T19:10:00+00:00",
      "date_modified": "2019-05-10T19:10:00+00:00",
      "author": {
        "name": "Alexandre Vallières-Lagacé",
        "url": "http://vallier.es"
      }
    },
    {
      "id": "962e46254d1527862c1d81574000e58b295aca8b",
      "title": "Logi Circle 2 Camera and Ecosystem Review",
      "summary": "The Logi Circle 2 security camera has a ton of features and a great accessory ecosystem that could make it the most versatile camera on the market!",
      "content_text": "There I am reviewing another security camera, but this one is peculiar. Most of the time a security camera is a standalone, all-inclusive thing that you set and forget. It has a single precise function and does it on day one until its last day. But what if you could get something modular? Something that can be moved around the house. This is where the Logi Circle 2 camera comes in the picture!DesignThis camera is as small as a hockey puck albeit with a conical shape, ...",
      "content_html": "<p>There I am reviewing another security camera, but this one is peculiar. Most of the time a security camera is a standalone, all-inclusive thing that you set and forget. It has a single precise function and does it on day one until its last day. But what if you could get something modular? Something that can be moved around the house. This is where the <a href=\\"https://hipsterpixel.co/r/az/B0711V3LSQ/logi+circle+2+wired\\">Logi Circle 2 camera</a> comes in the picture!</p>",
      "url": "https://hipsterpixel.co/2019/04/29/logi-circle-2-camera-and-ecosystem-review/",
      "image": "https://ad3d98360fa0de008220-e893b890b8e259a099f8456bf1578245.ssl.cf5.rackcdn.com/logi-circle-2-camera-review-573-c-3nsh3.jpg",
      "tags": [
        "logi",
        "logitech",
        "circle 2",
        "security camera",
        "camera",
        "video recording",
        "thieves",
        "caught on camera",
        "ecosystem",
        "accessories",
        "review",
        "test",
        "benchmark"
      ],
      "date_published": "2019-04-29T12:12:00+00:00",
      "date_modified": "2019-04-29T12:12:00+00:00",
      "author": {
        "name": "Alexandre Vallières-Lagacé",
        "url": "http://vallier.es"
      }
    },
    {
      "id": "946429b6cf42bd1691c3d2da50a9bceaa3aaa452",
      "title": "ElevationLab BatteryPro, a Battery Pack With Integrated Apple Watch Charger [Review]",
      "summary": "Take a battery pack, add a nice design and sprinkle original features on top, you get the BatteryPro by ElevationLab, let's see the result!",
      "content_text": "When we are talking about great design and battery pack, well, we are never mixing both. Most battery packs on the market are what they are, energy in a pack. But never, or extremely rarely, is it a work of product design. That was until, ElevationLab took a crack at it! With the BatteryPro, ElevationLab are cramming a big 8,000 mAh inside a nicely designed power brick with the added touch of having an Apple Watch charger built-in.",
      "content_html": "<p>When we are talking about great design and battery pack, well, we are never mixing both. Most battery packs on the market are what they are, energy in a pack. But never, or extremely rarely, is it a work of product design. That was until, ElevationLab took a crack at it! With the <a href=\\"https://www.elevationlab.com/products/battery-pro-for-iphone-apple-watch\\">BatteryPro</a>",
      "url": "https://hipsterpixel.co/2019/04/23/elevationlab-batterypro-a-battery-pack-with-integrated-apple-watch-charger-review/",
      "image": "https://ad3d98360fa0de008220-e893b890b8e259a099f8456bf1578245.ssl.cf5.rackcdn.com/elevationlab-batterypro-battery-pack-apple-watch-iphone-review-859-6i7yf.jpg",
      "tags": [
        "elevationlab",
        "batterypro",
        "battery pack",
        "external battery",
        "review",
        "test",
        "apple watch",
        "made for iphone"
      ],
      "date_published": "2019-04-23T12:20:00+00:00",
      "date_modified": "2019-04-23T12:20:00+00:00",
      "author": {
        "name": "Alexandre Vallières-Lagacé",
        "url": "http://vallier.es"
      }
    }
  ]
}''';
    Feed feed = parseFeed(meta);
    expect(feed.title, "My Entity Feed",
        reason: "The title should equal My Entity Feed");
    expect(feed.description, "I am the description of the entity",
        reason:
            "The description should equal I am the description of the entity");
    expect(feed.items.length, 2, reason: "The feed should have 2 items");
  });
}
